/*
    Gauztris, an svgalib Tetris-like game for consoles
    Copyright (C) 2001 Brian Mastrobuono <gauze@rifug.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <vga.h>
#include <vgagl.h>
#include <vgakeyboard.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <string.h>
#include "shapes.h"
#include "beep.h"

typedef int bool;
#define TRUE 1
#define FALSE 0

enum Colors { BLACK, BLUE, GREEN, CYAN, RED, MAGENTA, BROWN, GREY, DARK_GREY,
		LIGHT_BLUE, LIGHT_GREEN, LIGHT_CYAN, LIGHT_RED, LIGHT_MAGENTA,
		YELLOW, WHITE };

#define OFFSET 15 /* how far are we from left edge ... */

// defines related to blocks in 'grid'
#define NO_BLOCK 0
#define BLOCK 1
#define FINISHED_BLOCK 3

#define SCALE 8
// scale up 8* from ncurses version ....


struct Player {
	unsigned int score;
	unsigned int level;
	unsigned int lines;
} player;

struct Piece {
	int vert;
	int horiz;
	int shape[19][3][2];
	int shapetype;
	int shapenum;
} piece , mynext;

struct HighScore {
        unsigned int score;
        char name[11];
        unsigned int lines;
} temp, highscore[10];

int playfield[24][10];

bool PLAYING = TRUE; // main loop control
bool JUSTDROPPED = TRUE; // control thing  
bool AT_BOTTOM = FALSE; // internal checking
bool GOINGDOWN = TRUE; // this move is down?
bool SHOW_NEXT = TRUE; // for next box

/* globals ... yeah I know shut up */
int i, j, ch;
int oldposhoriz = 0;
int oldposvert = 0;
int oldshapetype = 0;
int cycle_count = 0;	
int startlevel = 1;
int baseshapetype;

#include "functions.h"

int main(int argc, char *argv[])
{
	int action = 0; // needed for key returns ...

	/* svgalib init */

	vga_init();
	vga_setmode(G320x200x256);
	gl_setcontextvga(G320x200x256);

	vga_setcolor(WHITE);
	gl_setfont(8, 8, gl_font8x8);
	gl_setwritemode(FONT_COMPRESSED + WRITEMODE_OVERWRITE);
	gl_setfontcolors(BLACK, WHITE);
	//gl_enableclipping();
	if (keyboard_init())
	{
		printf("Error: INIT_KEYB!\n");
		exit(1);
	}




	initfield(); 
		
	while( (ch = getopt(argc,argv,"l:")) != -1)
		if (ch == 'l')
			startlevel = atoi(optarg);
		

	player.level = startlevel;

	while ( PLAYING )
	{
		AT_BOTTOM = FALSE;
	
		draw();
		action = get_keys();

		oldposhoriz = piece.horiz; // next 3 needed for movevalid test	
		oldposvert = piece.vert;  // must stay here, after keyscan	
		oldshapetype = piece.shapetype; // before tests

		// key press return value functions with validation afterwards:
		if ( action == 32 )
		{	
			piece.vert++;	
			cycle_count = 0;	
			if ( !movevalid() )
			{
				piece.vert--;
				AT_BOTTOM = TRUE;
			}
		}
		else if ( action == 8 )
		{
			rotate_clock();
			if ( !movevalid() )
				rotate_counterclock();
		}
		else if ( action == -8 )
		{
			rotate_counterclock();
			if ( !movevalid() )
				rotate_clock();
		}
		else
		{		
			piece.horiz += action;
			if ( !movevalid() )
				piece.horiz -= action;
		}
	        verify_place();
		if (!AT_BOTTOM)
			timer(); // time based vertical movement
		verify_place();
		cycle_count++;
		if (AT_BOTTOM)
			blockdone();
		draw();

	}
	gameover();
	sleep(1);
	//hiscores();
	keyboard_close();
	vga_setmode(TEXT);

	return EXIT_SUCCESS;
}

